==> p5-Sys-Filesystem-1.21_1.tbz: lib/perl5/site_perl/5.8.8/Sys/Filesystem/Freebsd.pm <==
############################################################
#
#   $Id: Freebsd.pm 364 2006-03-23 15:22:19Z nicolaw $
#   Sys::Filesystem - Retrieve list of filesystems and their properties
#
#   Copyright 2004,2005,2006 Nicola Worthington
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
############################################################

package Sys::Filesystem::Freebsd;
# vim:ts=4:sw=4:tw=78

use strict;
use FileHandle;
use Carp qw(croak);

# For access to the getfsstat system call
require 'sys/syscall.ph';
require 'sys/mount.ph';

use vars qw($VERSION);
$VERSION = '1.05' || sprintf('%d', q$Revision: 364 $ =~ /(\d+)/g);

my $sizeof;
if ( &STATFS_VERSION == 0x20030518 ) {
	$sizeof = 472; # The size in bytes of the statfs structure
} else {
	croak "The statfs strucuture changed version (" . &STATFS_VERSION . ")\n";
}
# unpack format, we want the 3rd and the last 3 fields.
my $format = 'x8L' . 'x268' . 'A' . &MFSNAMELEN . ('A' . &MNAMELEN ) x 2 ;
  
sub new {
	ref(my $class = shift) && croak 'Class name required';
	my %args = @_;
	my $self = { };

	$args{fstab} ||= '/etc/fstab';
	# $args{mtab} ||= '/etc/mtab';  # Does not exist on FreeBSD
	$args{xtab} ||= '/var/db/mountdtab';

	my @keys = qw(fs_spec fs_file fs_vfstype fs_mntops fs_freq fs_passno);
	my @special_fs = qw(swap procfs devpts devfs tmpfs);

	# Read the fstab
	my $fstab = new FileHandle;
	if ($fstab->open($args{fstab})) {
		while (<$fstab>) {
			next if (/^\s*#/ || /^\s*$/);
			my @vals = split(/\s+/, $_);
			$self->{$vals[1]}->{mount_point} = $vals[1];
			$self->{$vals[1]}->{device} = $vals[0];
			$self->{$vals[1]}->{unmounted} = 1;
			$self->{$vals[1]}->{special} = 1 if grep(/^$vals[2]$/,@special_fs);
			for (my $i = 0; $i < @keys; $i++) {
				$self->{$vals[1]}->{$keys[$i]} = $vals[$i];
			}
		}
		$fstab->close;
	} else {
		croak "Unable to open fstab file ($args{fstab})\n";
	}

	# Get the number of mounted fileystems we have
	my $buf = '';
	my $cnt = syscall(&SYS_getfsstat, $buf, length $buf, &MNT_NOWAIT);

	# Fix a bug on some 5.x systems, the previous syscall may return 0...
	$cnt = 20 if ( ! $cnt );

	# Preallocate the buffer memory per the syscall() requreiments
	$buf = ' ' x ( $sizeof * $cnt );

	if ( ($cnt=syscall(&SYS_getfsstat, $buf, length $buf, &MNT_NOWAIT)) ) {
		for (my ($i) = 0; $i < $cnt; $i++) {

			my $offset = ($i)? 'x' . ($i * $sizeof): '';
			my @vals = unpack ( $offset . $format, $buf);

			delete $self->{$vals[3]}->{unmounted} if exists $self->{$vals[3]}->{unmounted};
			$self->{$vals[3]}->{mounted} = 1;
			$self->{$vals[3]}->{mount_point} = $vals[3];
			$self->{$vals[3]}->{device} = $vals[2];
			$self->{$vals[3]}->{special} = 1 if grep(/^$vals[1]$/,@special_fs);

			$self->{$vals[3]}->{fs_spec} = $vals[2];
			$self->{$vals[3]}->{fs_file} = $vals[3];
			$self->{$vals[3]}->{fs_vfstype} = $vals[1];
			$self->{$vals[3]}->{fs_mntops} = 
				($vals[0] & &MNT_RDONLY)? 'ro': 'rw';
			$self->{$vals[3]}->{fs_mntops} .= ',noexec'
				if ($vals[0] & &MNT_NOEXEC);
			$self->{$vals[3]}->{fs_mntops} .= ',nosuid'
				if ($vals[0] & &MNT_NOSUID);
			$self->{$vals[3]}->{fs_mntops} .= ',nodev'
				if ($vals[0] & &MNT_NODEV);
		}
	} else {
		croak "Unable to retrieve mounted filesystem information\n";
	}

	# Bless and return
	bless($self,$class);
	return $self;
}

1;


# See the fstab(5) manual page for important information on automatic mounts
# of network filesystems before modifying this file.
#
# Device                Mountpoint      FStype  Options         Dump    Pass#
#/dev/da0s1b             none            swap    sw              0       0
#/dev/da1s1b             none            swap    sw              0       0
#/dev/da0s1a             /               ufs     rw              1       1
#/dev/da1s1e             /home           ufs     rw              2       2
#/dev/da0s1e             /usr            ufs     rw              2       2
#/dev/da1s1f             /var            ufs     rw              2       2
#/dev/acd0c              /cdrom          cd9660  ro,noauto       0       0
#/var/tmp                /tmp            null    rw              0       0
#proc                    /proc           procfs  rw              0       0
#/etc/portal.conf        /p              portal  rw              0       0




###############################################################################
# POD

=pod

=head1 NAME

Sys::Filesystem::Freebsd - Return Freebsd filesystem information to Sys::Filesystem

=head1 SYNOPSIS

See L<Sys::Filesystem>.

=head1 METHODS

The following is a list of filesystem properties which may
be queried as methods through the parent L<Sys::Filesystem> object.

=over 4

=item fs_spec

Dscribes the block special device or remote filesystem to be mounted.

=item fs_file

Describes the mount point for the filesystem. For swap partitions,
this field should be specified as none. If the name of the mount
point contains spaces these can be escaped as \040.

=item fs_vfstype

Dscribes the type  of  the  filesystem.

=item fs_mntops

Describes the mount options associated with the filesystem.

=item fs_freq

Used  for  these filesystems by the
L<dump(8)> command to determine which filesystems need to be  dumped.

=item fs_passno

Used by the L<fsck(8)> program to  determine the order in which filesystem
checks are done at reboot time. 

=back

=head1 SEE ALSO

L<Sys::Filesystem>, L<Sys::Filesystem::Unix>, L<fstab(5)>

=head1 VERSION

$Id: Freebsd.pm 364 2006-03-23 15:22:19Z nicolaw $

=head1 AUTHOR

Nicola Worthington <nicolaw@cpan.org>

L<http://perlgirl.org.uk>

=head1 COPYRIGHT

Copyright 2004,2005,2006 Nicola Worthington.

This software is licensed under The Apache Software License, Version 2.0.

L<http://www.apache.org/licenses/LICENSE-2.0>

=cut

==> p5-Sys-Filesystem-1.22.tbz: lib/perl5/site_perl/5.8.8/Sys/Filesystem/Freebsd.pm <==
############################################################
#
#   $Id: Freebsd.pm 364 2006-03-23 15:22:19Z nicolaw $
#   Sys::Filesystem - Retrieve list of filesystems and their properties
#
#   Copyright 2004,2005,2006 Nicola Worthington
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
############################################################

package Sys::Filesystem::Freebsd;
# vim:ts=4:sw=4:tw=78

use strict;
use FileHandle;
use Carp qw(croak);

# For access to the getfsstat system call
require 'sys/syscall.ph';
require 'sys/mount.ph';

use vars qw($VERSION);
$VERSION = '1.05' || sprintf('%d', q$Revision: 364 $ =~ /(\d+)/g);

my $sizeof;
if ( &STATFS_VERSION == 0x20030518 ) {
	$sizeof = 472; # The size in bytes of the statfs structure
} else {
	croak "The statfs strucuture changed version (" . &STATFS_VERSION . ")\n";
}
# unpack format, we want the 3rd and the last 3 fields.
my $format = 'x8L' . 'x268' . 'A' . &MFSNAMELEN . ('A' . &MNAMELEN ) x 2 ;
  
sub new {
	ref(my $class = shift) && croak 'Class name required';
	my %args = @_;
	my $self = { };
	my (@vals, $i);

	$args{fstab} ||= '/etc/fstab';
	# $args{mtab} ||= '/etc/mtab';  # Does not exist on FreeBSD
	$args{xtab} ||= '/var/db/mountdtab';

	my @keys = qw(fs_spec fs_file fs_vfstype fs_mntops fs_freq fs_passno);
	push (@Sys::Filesystem::special_fs, qw(procfs devpts devfs));

	# Read the fstab
	my $fstab = new FileHandle;
	if ($fstab->open($args{fstab})) {
		while (<$fstab>) {
			next if (/^\s*#/ || /^\s*$/);
			@vals = split(/\s+/, $_);
			$self->{$vals[1]}->{mount_point} = $vals[1];
			$self->{$vals[1]}->{device} = $vals[0];
			$self->{$vals[1]}->{unmounted} = 1;
			$self->{$vals[1]}->{special} = 1 if grep(/^$vals[2]$/,@Sys::Filesystem::special_fs);
			for ($i = 0; $i < @keys; $i++) {
				$self->{$vals[1]}->{$keys[$i]} = $vals[$i];
			}
		}
		$fstab->close;
	} else {
		croak "Unable to open fstab file ($args{fstab})\n";
	}

	# Get the number of mounted fileystems we have
	my $buf = '';
	my $cnt = syscall(&SYS_getfsstat, $buf, length $buf, &MNT_NOWAIT);

	# Fix a bug on some 5.x systems, the previous syscall may return 0...
	$cnt = 20 if ( ! $cnt );

	# Preallocate the buffer memory per the syscall() requreiments
	$buf = ' ' x ( $sizeof * $cnt );

	if ( ($cnt=syscall(&SYS_getfsstat, $buf, length $buf, &MNT_NOWAIT)) ) {
		for (($i) = 0; $i < $cnt; $i++) {

			my $offset = ($i)? 'x' . ($i * $sizeof): '';
			@vals = unpack ( $offset . $format, $buf);

			delete $self->{$vals[3]}->{unmounted} if exists $self->{$vals[3]}->{unmounted};
			$self->{$vals[3]}->{mounted} = 1;
			$self->{$vals[3]}->{mount_point} = $vals[3];
			$self->{$vals[3]}->{device} = $vals[2];
			$self->{$vals[3]}->{special} = 1 if grep(/^$vals[1]$/,@Sys::Filesystem::special_fs);

			$self->{$vals[3]}->{fs_spec} = $vals[2];
			$self->{$vals[3]}->{fs_file} = $vals[3];
			$self->{$vals[3]}->{fs_vfstype} = $vals[1];
			$self->{$vals[3]}->{fs_mntops} = 
				($vals[0] & &MNT_RDONLY)? 'ro': 'rw';
			$self->{$vals[3]}->{fs_mntops} .= ',noexec'
				if ($vals[0] & &MNT_NOEXEC);
			$self->{$vals[3]}->{fs_mntops} .= ',nosuid'
				if ($vals[0] & &MNT_NOSUID);
			$self->{$vals[3]}->{fs_mntops} .= ',nodev'
				if ($vals[0] & &MNT_NODEV);
		}
	} else {
		croak "Unable to retrieve mounted filesystem information\n";
	}

	# Bless and return
	bless($self,$class);
	return $self;
}

1;


# See the fstab(5) manual page for important information on automatic mounts
# of network filesystems before modifying this file.
#
# Device                Mountpoint      FStype  Options         Dump    Pass#
#/dev/da0s1b             none            swap    sw              0       0
#/dev/da1s1b             none            swap    sw              0       0
#/dev/da0s1a             /               ufs     rw              1       1
#/dev/da1s1e             /home           ufs     rw              2       2
#/dev/da0s1e             /usr            ufs     rw              2       2
#/dev/da1s1f             /var            ufs     rw              2       2
#/dev/acd0c              /cdrom          cd9660  ro,noauto       0       0
#/var/tmp                /tmp            null    rw              0       0
#proc                    /proc           procfs  rw              0       0
#/etc/portal.conf        /p              portal  rw              0       0




###############################################################################
# POD

=pod

=head1 NAME

Sys::Filesystem::Freebsd - Return Freebsd filesystem information to Sys::Filesystem

=head1 SYNOPSIS

See L<Sys::Filesystem>.

=head1 METHODS

The following is a list of filesystem properties which may
be queried as methods through the parent L<Sys::Filesystem> object.

=over 4

=item fs_spec

Dscribes the block special device or remote filesystem to be mounted.

=item fs_file

Describes the mount point for the filesystem. For swap partitions,
this field should be specified as none. If the name of the mount
point contains spaces these can be escaped as \040.

=item fs_vfstype

Dscribes the type  of  the  filesystem.

=item fs_mntops

Describes the mount options associated with the filesystem.

=item fs_freq

Used  for  these filesystems by the
L<dump(8)> command to determine which filesystems need to be  dumped.

=item fs_passno

Used by the L<fsck(8)> program to  determine the order in which filesystem
checks are done at reboot time. 

=back

=head1 SEE ALSO

L<Sys::Filesystem>, L<Sys::Filesystem::Unix>, L<fstab(5)>

=head1 VERSION

$Id: Freebsd.pm 364 2006-03-23 15:22:19Z nicolaw $

=head1 AUTHOR

Nicola Worthington <nicolaw@cpan.org>

L<http://perlgirl.org.uk>

=head1 COPYRIGHT

Copyright 2004,2005,2006 Nicola Worthington.

This software is licensed under The Apache Software License, Version 2.0.

L<http://www.apache.org/licenses/LICENSE-2.0>

=cut
